.dashboard > .container > .row > .columns {
    margin-bottom: 2em;
}

.partial {
    border-radius: 6px;
    border: 1px solid #eee;
    padding: 8px;
}

.page-title {
    color:#008080;
}

.logo-image {
    text-align: center;
    max-width: 50px;
    width: 100%;
    height: auto;
}

.hero {
    background-color: #F7F7F5;
    width: 100%;
}

a {
    text-decoration: none;
}

.alert-partial.partial:hover {
    cursor: pointer;   
}
.close-icon {
    float: right;
    color: black;
    padding-right: 5px;
}

.navbar {
    background-color: black;
    width:100%;
    margin-bottom: 2rem;
}

.navbar-item {
    margin-top: 1rem;
    color: white;
    border: none;
}

.table-partial > table > thead > tr, tbody {
    /* display:block; */
}
.table-partial > table > tbody > tr:hover { 
    background: #F7F7F5;
    cursor: pointer;
}