<!DOCTYPE html>
<html lang="en">
<head>

  <!-- Basic Page Needs
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta charset="utf-8">
  <title>Dashboards</title>
  <meta name="description" content="">
  <meta name="author" content="Udit Vira">

  <!-- Mobile Specific Metas
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <!-- FONT
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link href='//fonts.googleapis.com/css?family=Raleway:400,300,600' rel='stylesheet' type='text/css'>

  <!-- CSS
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="stylesheet" type="text/css" href='/static/normalize.css'>
  <link rel="stylesheet" type="text/css" href='/static/skeleton.css'>
  <link rel="stylesheet" type="text/css" href='/static/custom.css'>
  <link rel="stylesheet" href="https://openlayers.org/en/v4.6.5/css/ol.css" type="text/css">

  <!-- Scripts
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <script src="//ajax.googleapis.com/ajax/libs/jquery/2.1.1/jquery.min.js"></script> 
  <script src="https://cdn.socket.io/socket.io-1.2.0.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/Chart.js/2.7.2/Chart.min.js"></script>
  <script src="https://openlayers.org/en/v4.6.5/build/ol.js" type="text/javascript"></script>
  <script src='/static/main.js'></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.22.0/moment.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/string-format/0.5.0/string-format.min.js"></script>
  <!-- TODO Favicon
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
  <link rel="shortcut icon" type="image/png" href="/static/favicon.ico"/>


</head>
<body>
<!-- Start Body
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
    <section class="section hero">
      <div class="container">
          <div class="row spacer">
              <br>
          </div>
          <div class="row">
              <div class="one columns">
                <a href="/">
                  <img class="logo-image" src='/static/link.svg'>
                </a>
              </div>
              <div class="eleven columns">
                <a href="/">
                  <h4 class="page-title"> Steamlink Dashboard </h4>
                </a>
              </div>
          </div>
      </div>
    </section>

    <section class="section navbar">
      {% include 'navbar.html' %}
    </section>

    <section class="section dashboard">
      <div class="container">
        <div class="alert-partial partial" id="dashboard_socket_alerts" style="display:none;">
          <div class="alert-msg" style="display:inline-block"></div>
          <a class="alert-partial close-icon" style="display:inline-block" ></a>
        </div>
      </div>
    </section>

    <section class="section dashboard">
        <div class="container">
            {% for row in context %}
              <div class="row">
                  {% for partial in context[row] %}
                    {% with partial_item=partial %}
                      {% include partial.template %}
                    {% endwith %}                    
                  {% endfor %}
              </div>
            {% endfor %}
        </div>
    </section>

<!-- End Document
  –––––––––––––––––––––––––––––––––––––––––––––––––– -->
</body>
</html>