<div class="container">
    <div class="row">
        <div class="twelve columns">
            {% for button in navbar %}
                <a class="button navbar-item" href="{{button}}">{{ button | capitalize }}</a>
            {% endfor %}
        </div>
    </div>
</div>